@@ -3,7 +3,7 @@
  * \brief YASM value/parameter interface.
  *
  * \rcs
- * $Id: valparam.h 1876 2007-06-02 21:50:51Z peter $
+ * $Id: valparam.h 2101 2008-05-23 06:46:51Z peter $
  * \endrcs
  *
  * \license
@@ -34,36 +34,40 @@
 #ifndef YASM_VALPARAM_H
 #define YASM_VALPARAM_H
 
-#ifdef YASM_LIB_INTERNAL
+#ifndef YASM_LIB_DECL
+#define YASM_LIB_DECL
+#endif
+
 /** Value/parameter pair.  \internal */
 struct yasm_valparam {
     /*@reldef@*/ STAILQ_ENTRY(yasm_valparam) link;  /**< Next pair in list */
-    /*@owned@*/ /*@null@*/ char *val;		/**< Value */
+    /*@owned@*/ /*@null@*/ char *val;           /**< Value */
 
+    /** Parameter type. */
     enum yasm_param_type {
         YASM_PARAM_ID,                          /**< Identifier */
         YASM_PARAM_STRING,                      /**< String */
         YASM_PARAM_EXPR                         /**< Expression */
     } type;                                     /**< Parameter type */
 
+    /** Parameter value. */
     union yasm_param {
         /*@owned@*/ char *id;                   /**< Identifier */
         /*@owned@*/ char *str;                  /**< String */
         /*@owned@*/ yasm_expr *e;               /**< Expression */
     } param;                                    /**< Parameter */
 
-    /* Prefix character that indicates a raw identifier.  When yasm_vp_string()
-     * is called on a #YASM_PARAM_ID, all characters are returned.  When
-     * yasm_vp_id() is called on a #YASM_PARAM_ID, if the identifier begins
-     * with this character, this character is stripped from the returned
-     * value.
+    /** Prefix character that indicates a raw identifier.  When
+     * yasm_vp_string() is called on a #YASM_PARAM_ID, all characters are
+     * returned.  When yasm_vp_id() is called on a #YASM_PARAM_ID, if the
+     * identifier begins with this character, this character is stripped
+     * from the returned value.
      */
     char id_prefix;
 };
 
 /** Linked list of value/parameter pairs.  \internal */
 /*@reldef@*/ STAILQ_HEAD(yasm_valparamhead, yasm_valparam);
-#endif
 
 /** Directive list entry structure. */
 struct yasm_directive {
@@ -73,45 +77,47 @@ struct yasm_directive {
      */
     /*@null@*/ const char *name;
 
-    const char *parser;			    /**< Parser keyword */
+    const char *parser;                     /**< Parser keyword */
 
     /** Handler callback function for the directive.
-     * \param object		object 
-     * \param valparams		value/parameters
-     * \param objext_valparams	object format-specific value/parameters
-     * \param line		virtual line (from yasm_linemap)
+     * \param object            object 
+     * \param valparams         value/parameters
+     * \param objext_valparams  object format-specific value/parameters
+     * \param line              virtual line (from yasm_linemap)
      */
     void (*handler) (yasm_object *object, yasm_valparamhead *valparams,
-		     yasm_valparamhead *objext_valparams, unsigned long line);
+                     yasm_valparamhead *objext_valparams, unsigned long line);
 
     /** Flags for pre-handler parameter checking. */
     enum yasm_directive_flags {
-	YASM_DIR_ANY = 0,	    /**< Any valparams accepted */
-	YASM_DIR_ARG_REQUIRED = 1,  /**< Require at least 1 valparam */
-	YASM_DIR_ID_REQUIRED = 2    /**< First valparam must be ID */
+        YASM_DIR_ANY = 0,           /**< Any valparams accepted */
+        YASM_DIR_ARG_REQUIRED = 1,  /**< Require at least 1 valparam */
+        YASM_DIR_ID_REQUIRED = 2    /**< First valparam must be ID */
     } flags;
 };
 
 /** Call a directive.  Performs any valparam checks asked for by the
  * directive prior to call.  Note that for a variety of reasons, a directive
  * can generate an error.
- * \param directive		directive
- * \param object		object 
- * \param valparams		value/parameters
- * \param objext_valparams	object format-specific value/parameters
- * \param line			virtual line (from yasm_linemap)
+ * \param directive             directive
+ * \param object                object 
+ * \param valparams             value/parameters
+ * \param objext_valparams      object format-specific value/parameters
+ * \param line                  virtual line (from yasm_linemap)
  */
+YASM_LIB_DECL
 void yasm_call_directive(const yasm_directive *directive, yasm_object *object,
-			 yasm_valparamhead *valparams,
-			 yasm_valparamhead *objext_valparams,
-			 unsigned long line);
+                         yasm_valparamhead *valparams,
+                         yasm_valparamhead *objext_valparams,
+                         unsigned long line);
 
 /** Create a new valparam with identifier parameter.
- * \param v	value
- * \param p	parameter
+ * \param v             value
+ * \param p             parameter
  * \param id_prefix     identifier prefix for raw identifiers
  * \return Newly allocated valparam.
  */
+YASM_LIB_DECL
 yasm_valparam *yasm_vp_create_id(/*@keep@*/ char *v, /*@keep@*/ char *p,
                                  int id_prefix);
 
@@ -120,6 +126,7 @@ yasm_valparam *yasm_vp_create_id(/*@keep@*/ char *v, /*@keep@*/ char *p,
  * \param p     parameter
  * \return Newly allocated valparam.
  */
+YASM_LIB_DECL
 yasm_valparam *yasm_vp_create_string(/*@keep@*/ char *v, /*@keep@*/ char *p);
 
 /** Create a new valparam with expression parameter.
@@ -127,17 +134,19 @@ yasm_valparam *yasm_vp_create_string(/*@keep@*/ char *v, /*@keep@*/ char *p);
  * \param p     parameter
  * \return Newly allocated valparam.
  */
+YASM_LIB_DECL
 yasm_valparam *yasm_vp_create_expr(/*@keep@*/ char *v,
                                    /*@keep@*/ yasm_expr *p);
 
 /** Get a valparam parameter as an expr.  If the parameter is an identifier,
  * it's treated as a symbol (yasm_symtab_use() is called to convert it).
- * \param vp		valparam
- * \param symtab	symbol table
- * \param line		virtual line
+ * \param vp            valparam
+ * \param symtab        symbol table
+ * \param line          virtual line
  * \return Expression, or NULL if vp is NULL or the parameter cannot be
  *         converted to an expression.
  */
+YASM_LIB_DECL
 /*@null@*/ /*@only@*/ yasm_expr *yasm_vp_expr
     (const yasm_valparam *vp, yasm_symtab *symtab, unsigned long line);
 
@@ -147,6 +156,7 @@ yasm_valparam *yasm_vp_create_expr(/*@keep@*/ char *v,
  * \return String, or NULL if vp is NULL or the parameter cannot be realized
  *         as a string.
  */
+YASM_LIB_DECL
 /*@null@*/ /*@dependent@*/ const char *yasm_vp_string(const yasm_valparam *vp);
 
 /** Get a valparam parameter as an identifier.
@@ -154,74 +164,82 @@ yasm_valparam *yasm_vp_create_expr(/*@keep@*/ char *v,
  * \return Identifier (string), or NULL if vp is NULL or the parameter is not
  *         an identifier.
  */
+YASM_LIB_DECL
 /*@null@*/ /*@dependent@*/ const char *yasm_vp_id(const yasm_valparam *vp);
 
 /** Create a new linked list of valparams.
  * \return Newly allocated valparam list.
  */
+YASM_LIB_DECL
 yasm_valparamhead *yasm_vps_create(void);
 
 /** Destroy a list of valparams (created with yasm_vps_create).
- * \param headp		list of valparams
+ * \param headp         list of valparams
  */
+YASM_LIB_DECL
 void yasm_vps_destroy(yasm_valparamhead *headp);
 
-#ifdef YASM_LIB_INTERNAL
 /** Initialize linked list of valparams.
- * \param headp	linked list
+ * \param headp linked list
  */
-#define yasm_vps_initialize(headp)	STAILQ_INIT(headp)
+void yasm_vps_initialize(/*@out@*/ yasm_valparamhead *headp);
+#ifndef YASM_DOXYGEN
+#define yasm_vps_initialize(headp)      STAILQ_INIT(headp)
+#endif
 
 /** Destroy (free allocated memory for) linked list of valparams (created with
  * yasm_vps_initialize).
  * \warning Deletes val/params.
- * \param headp	linked list
+ * \param headp linked list
  */
+YASM_LIB_DECL
 void yasm_vps_delete(yasm_valparamhead *headp);
-#endif
 
 /** Append valparam to tail of linked list.
- * \param headp	linked list
- * \param vp	valparam
+ * \param headp linked list
+ * \param vp    valparam
  */
 void yasm_vps_append(yasm_valparamhead *headp, /*@keep@*/ yasm_valparam *vp);
-#ifdef YASM_LIB_INTERNAL
-#define yasm_vps_append(headp, vp)	do {	    \
-	if (vp)					    \
-	    STAILQ_INSERT_TAIL(headp, vp, link);    \
+#ifndef YASM_DOXYGEN
+#define yasm_vps_append(headp, vp)      do {        \
+        if (vp)                                     \
+            STAILQ_INSERT_TAIL(headp, vp, link);    \
     } while(0)
 #endif
 
 /** Get first valparam in linked list.
- * \param headp	linked list
+ * \param headp linked list
  * \return First valparam in linked list.
  */
 /*@null@*/ /*@dependent@*/ yasm_valparam *yasm_vps_first
     (yasm_valparamhead *headp);
-#ifdef YASM_LIB_INTERNAL
-#define yasm_vps_first(headp)	    STAILQ_FIRST(headp)
+#ifndef YASM_DOXYGEN
+#define yasm_vps_first(headp)       STAILQ_FIRST(headp)
 #endif
 
 /** Get next valparam in linked list.
- * \param cur	previous valparam in linked list
+ * \param cur   previous valparam in linked list
  * \return Next valparam in linked list.
  */
 /*@null@*/ /*@dependent@*/ yasm_valparam *yasm_vps_next(yasm_valparam *cur);
-#ifdef YASM_LIB_INTERNAL
-#define yasm_vps_next(cur)	    STAILQ_NEXT(cur, link)
+#ifndef YASM_DOXYGEN
+#define yasm_vps_next(cur)          STAILQ_NEXT(cur, link)
+#endif
 
 /** Iterate through linked list of valparams.
  * \internal
- * \param iter	    iterator variable
- * \param headp	    linked list
+ * \param iter      iterator variable
+ * \param headp     linked list
  */
-#define yasm_vps_foreach(iter, headp)	STAILQ_FOREACH(iter, headp, link)
+#ifndef YASM_DOXYGEN
+#define yasm_vps_foreach(iter, headp)   STAILQ_FOREACH(iter, headp, link)
 #endif
 
 /** Print linked list of valparams.  For debugging purposes.
- * \param f	file
- * \param headp	linked list
+ * \param f     file
+ * \param headp linked list
  */
+YASM_LIB_DECL
 void yasm_vps_print(/*@null@*/ const yasm_valparamhead *headp, FILE *f);
 
 /** Directive valparam parse helper structure. */
@@ -277,6 +295,7 @@ typedef struct yasm_dir_help {
  * \return -1 on error, 1 if any arguments matched (including via
  *         catch-all callback), 0 if no match.
  */
+YASM_LIB_DECL
 int yasm_dir_helper(void *obj, yasm_valparam *vp_first, unsigned long line,
                     const yasm_dir_help *help, size_t nhelp, void *data,
                     int (*helper_valparam) (void *object,
@@ -294,6 +313,7 @@ int yasm_dir_helper(void *obj, yasm_valparam *vp_first, unsigned long line,
  * \param arg   flag to set
  * \return 0
  */
+YASM_LIB_DECL
 int yasm_dir_helper_flag_set(void *obj, yasm_valparam *vp, unsigned long line,
                              void *data, uintptr_t arg);
 
@@ -307,6 +327,7 @@ int yasm_dir_helper_flag_set(void *obj, yasm_valparam *vp, unsigned long line,
  * \param arg   flag to OR
  * \return 0
  */
+YASM_LIB_DECL
 int yasm_dir_helper_flag_or(void *obj, yasm_valparam *vp, unsigned long line,
                             void *data, uintptr_t arg);
 
@@ -320,9 +341,28 @@ int yasm_dir_helper_flag_or(void *obj, yasm_valparam *vp, unsigned long line,
  * \param arg   flag to AND
  * \return 0
  */
+YASM_LIB_DECL
 int yasm_dir_helper_flag_and(void *obj, yasm_valparam *vp, unsigned long line,
                              void *data, uintptr_t arg);
 
+/** Standard helper for yasm_dir_helper() that parses an expr parameter.
+ * The #yasm_dir_help structure that uses this function should have
+ * needsparam=1.  The obj parameter to yasm_dir_helper() when this helper
+ * is used MUST point to a #yasm_object.  In addition, the data parameter
+ * that is ultimately passed to this function (e.g. yasm_dir_helper() data
+ * parameter plus #yasm_dir_help.off) must point to a #yasm_expr *
+ * initialized to NULL.
+ * \param obj   object; must be #yasm_object
+ * \param vp    valparam
+ * \param line  virtual line number
+ * \param data  pointer to #yasm_expr *
+ * \param arg   unused argument
+ * \return -1 on error, 0 otherwise.
+ */
+YASM_LIB_DECL
+int yasm_dir_helper_expr(void *obj, yasm_valparam *vp, unsigned long line,
+                         void *data, uintptr_t arg);
+
 /** Standard helper for yasm_dir_helper() that parses an intnum parameter.
  * The #yasm_dir_help structure that uses this function should have
  * needsparam=1.  The obj parameter to yasm_dir_helper() when this helper
@@ -337,6 +377,7 @@ int yasm_dir_helper_flag_and(void *obj, yasm_valparam *vp, unsigned long line,
  * \param arg   unused argument
  * \return -1 on error, 0 otherwise.
  */
+YASM_LIB_DECL
 int yasm_dir_helper_intn(void *obj, yasm_valparam *vp, unsigned long line,
                          void *data, uintptr_t arg);
 
@@ -353,6 +394,7 @@ int yasm_dir_helper_intn(void *obj, yasm_valparam *vp, unsigned long line,
  * \param arg   unused
  * \return -1 on error, 0 otherwise.
  */
+YASM_LIB_DECL
 int yasm_dir_helper_string(void *obj, yasm_valparam *vp, unsigned long line,
                            void *data, uintptr_t arg);
 
@@ -364,6 +406,7 @@ int yasm_dir_helper_string(void *obj, yasm_valparam *vp, unsigned long line,
  * \param data  unused
  * \return 0
  */
+YASM_LIB_DECL
 int yasm_dir_helper_valparam_warn(void *obj, yasm_valparam *vp,
                                   unsigned long line, void *data);
 #endif
