@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: raw-preproc.c 2008 2007-11-03 04:37:44Z peter $");
+/*@unused@*/ RCSID("$Id: raw-preproc.c 2172 2009-01-27 06:38:14Z peter $");
 
 #include <libyasm.h>
 
@@ -94,7 +94,7 @@ raw_preproc_get_line(yasm_preproc *preproc)
         p += strlen(p);
         if (p > buf && p[-1] == '\n')
             break;
-        if ((p-buf) >= bufsize) {
+        if ((p-buf)+1 >= bufsize) {
             /* Increase size of buffer */
             char *oldbuf = buf;
             bufsize *= 2;
@@ -147,6 +147,12 @@ raw_preproc_define_builtin(yasm_preproc *preproc, const char *macronameval)
     /* no builtin defines */
 }
 
+static void
+raw_preproc_add_standard(yasm_preproc *preproc, const char **macros)
+{
+    /* no standard macros */
+}
+
 
 /* Define preproc structure -- see preproc.h for details */
 yasm_preproc_module yasm_raw_LTX_preproc = {
@@ -159,5 +165,6 @@ yasm_preproc_module yasm_raw_LTX_preproc = {
     raw_preproc_add_include_file,
     raw_preproc_predefine_macro,
     raw_preproc_undefine_macro,
-    raw_preproc_define_builtin
+    raw_preproc_define_builtin,
+    raw_preproc_add_standard
 };
